@@ -85,7 +85,8 @@ almanah_calendar_appointment_event_new (const gchar *summary, GTime start_time)
 	event->priv->start_time = start_time;
 
 	gmtime_r ((const time_t*) &(ALMANAH_CALENDAR_APPOINTMENT_EVENT (event)->priv->start_time), &utc_date_tm);
-	event->priv->time = g_strdup_printf ("%.2d:%.2d", utc_date_tm.tm_hour, utc_date_tm.tm_min);
+	/* Translators: This is a time string with the format hh:mm */
+	event->priv->time = g_strdup_printf (_("%.2d:%.2d"), utc_date_tm.tm_hour, utc_date_tm.tm_min);
 
 	return event;
 }
